/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import static java.lang.Math.max;
import static org.fortiss.tooling.base.ui.editor.fx.controller.ContextMenuUtil.createDisplayMenu;
import static org.fortiss.tooling.base.ui.editor.fx.controller.ContextMenuUtil.createElementCompositionContext;
import static org.fortiss.tooling.base.ui.editor.fx.controller.ContextMenuUtil.createPrototypeMenu;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewerFeatures;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.ControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IMVCBundle;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;

import javafx.scene.Node;
import javafx.scene.control.MenuItem;

/**
 * Diagram controller that uses the {@link IElementCompositorService} to create context menu
 * entries for prototypes provided by the {@link IContextMenuService}.
 * 
 * @author hoelzl
 */
public class EObjectDiagramController<T extends EObject> extends ControllerBase {

	/** The model change provider. */
	private final EObjectModelChangeProvider modelChangeProvider;

	/** Constructor. */
	public EObjectDiagramController(IMVCBundle mvcb, Class<T> modelType) {
		super(mvcb);
		Object model = mvcb.getModel();
		if(model == null) {
			throw new IllegalArgumentException("The given model is null!");
		}
		if(!modelType.isAssignableFrom(model.getClass())) {
			throw new IllegalArgumentException("Expected model of type " +
					modelType.getSimpleName() + ", but was " + model.getClass().getSimpleName());
		}
		this.modelChangeProvider = new EObjectModelChangeProvider(getModelElement());
	}

	/** {@inheritDoc} */
	@Override
	public IModelChangeProvider getModelChangeProvider() {
		return modelChangeProvider;
	}

	/** Returns the {@link EObject} model element. */
	@SuppressWarnings("unchecked")
	protected T getModelElement() {
		// wild cast works: see constructor check with exception
		return (T)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public List<MenuItem> contextMenuContributions(Node node, DiagramCoordinate diagramLocation) {
		// outer-most grid areas of the diagram are inaccessible
		DiagramViewerFeatures features = getViewer().getFeatures();
		double x = max(features.getHorizontalSpacing(), diagramLocation.getX());
		double y = max(features.getVerticalSpacing(), diagramLocation.getY());

		// wild cast works: see constructor exception
		T modelParent = getModelElement();
		IElementCompositionContext edc = createElementCompositionContext(modelParent, x, y, true,
				getViewer().getFeatures().getCurrentZoomFactor());
		List<MenuItem> result = createPrototypeMenu(modelParent, edc);
		result.add(createDisplayMenu(getViewer()));
		return result;
	}
}
